ulx.convar( "logEcho", "2", "", ULib.ACCESS_NONE )
ulx.convar( "logFile", "1", "", ULib.ACCESS_NONE )
ulx.convar( "logEvents", "1", "", ULib.ACCESS_NONE )
ulx.convar( "logChat", "1", "", ULib.ACCESS_NONE )
ulx.convar( "logDir", "ulx_logs", "", ULib.ACCESS_NONE )

function ulx.logUserAct( ply, target, action, hide_echo )
	local nick
	if ply:IsValid() then
		if not ply:IsConnected() or not target:IsConnected() then return end
		nick = ply:Nick()
	else
		nick = "(Console)"
	end

	action = action:gsub( "#T", target:Nick(), 1 ) -- Everything needs this replacement

	if not hide_echo then
		local level = GetConVarNumber( "ulx_logEcho" )
		if level == 1 then
			local echo = action:gsub( "#A", "(ADMIN)", 1 )
			ULib.tsay( _, echo )
		elseif level == 2 then
			local echo = action:gsub( "#A", "(ADMIN) " .. nick, 1 )
			ULib.tsay( _, echo )
		end
	else
		local echo = action:gsub( "#A", "(ADMIN) " .. nick, 1 )
		local players = player.GetAll()
		for _, player in ipairs( players ) do
			if player:IsUserGroup( ULib.ACCESS_SUPERADMIN ) then
				ULib.tsay( player, echo )
			end
		end
	end
	
	if isDedicatedServer() then
		Msg( action:gsub( "#A", "(ADMIN) " .. nick, 1 ) .. "\n" )
	end

	if util.tobool( GetConVarNumber( "ulx_logFile" ) ) then
		ulx.logWriteln( action:gsub( "#A", "(ADMIN) " .. nick, 1 ) )
	end
end

function ulx.logServAct( ply, action, hide_echo )
	local nick
	if ply:IsValid() then
		if not ply:IsConnected() then return end
		nick = ply:Nick()
	else
		nick = "(Console)"
	end
	
	if not hide_echo then
		local level = GetConVarNumber( "ulx_logEcho" )
		if level == 1 then
			local echo = action:gsub( "#A", "(ADMIN)", 1 )
			ULib.tsay( _, echo )
		elseif level == 2 then
			local echo = action:gsub( "#A", "(ADMIN) " .. nick, 1 )
			ULib.tsay( _, echo )
		end
	else
		local echo = action:gsub( "#A", "(ADMIN) " .. nick, 1 )
		local players = player.GetAll()
		for _, player in ipairs( players ) do
			if player:IsUserGroup( ULib.ACCESS_SUPERADMIN ) then
				ULib.tsay( player, echo )
			end
		end
	end
	
	if isDedicatedServer() then
		Msg( action:gsub( "#A", "(ADMIN) " .. nick, 1 ) .. "\n" )
	end

	if util.tobool( GetConVarNumber( "ulx_logFile" ) ) then
		ulx.logWriteln( action:gsub( "#A", "(ADMIN) " .. nick, 1 ) )
	end
end

function ulx.logString( str )
	ulx.logWriteln( str )
end

-- Utility stuff for our logs...
ulx.log_file = nil
local function init()
	ulx.log_file = os.date( GetConVarString( "ulx_logDir" ) .. "/" .. "%m-%d-%y" .. ".txt" )
	if not file.Exists( ulx.log_file ) then
		file.Write( ulx.log_file, "" )
	else
		ulx.logWriteln( "\n\n" ) -- Make some space
	end
	ulx.logWriteln( "New map: " .. game.GetMap() )
end
timer.Simple( 0.001, init) -- So there's time to load the settings

function ulx.logWriteln( str )
	if util.tobool( GetConVarNumber( "ulx_logFile" ) ) and ulx.log_file then
		file.Write( ulx.log_file, file.Read( ulx.log_file ) .. os.date( "[%H:%M:%S] " ) .. str .. "\n" )
	end
end

local function playerSay( ply, text, public )
	if util.tobool( GetConVarNumber( "ulx_logChat" ) ) then
		if not public then
			ulx.logWriteln( string.format( "(TEAM) %s: %s", ply:Nick(), text ) )
		else
			ulx.logWriteln( string.format( "%s: %s", ply:Nick(), text ) )
		end
	end
	return nil
end
hook.Add( "PlayerSay", "ULXLogSay", playerSay )

local function playerConnect( name, address, steamid )
	if util.tobool( GetConVarNumber( "ulx_logEvents" ) ) then
		ulx.logWriteln( string.format( "Client \"%s\" connected (%s).", name, address ) )
	end
end
hook.Add( "PlayerConnect", "ULXLogConnect", playerConnect )

local function playerDisconnect( ply )
	if util.tobool( GetConVarNumber( "ulx_logEvents" ) ) then
		ulx.logWriteln( string.format( "Dropped \"%s\" from server", ply:Nick() ) )
	end
end
hook.Add( "PlayerDisconnected", "ULXLogDisconnect", playerDisconnect )

local function playerDeath( victim, weapon, killer )
	if util.tobool( GetConVarNumber( "ulx_logEvents" ) ) then
		if not killer:IsPlayer() then
			ulx.logWriteln( string.format( "%s was killed by %s", victim:Nick(), killer:GetClass() ) )
		else
			if victim ~= killer then
				ulx.logWriteln( string.format( "%s killed %s using %s", killer:Nick(), victim:Nick(), weapon:GetClass() ) )
			else
				ulx.logWriteln( string.format( "%s suicided!", victim:Nick() ) )
			end
		end
	end
end
hook.Add( "PlayerDeath", "ULXLogDeath", playerDeath )
